#!/usr/bin/env python
# encoding: utf-8

import os
import errno

from moodlefuse.filesystem import Filesystem
from moodlefuse.core import setup, config

from moodlefuse.services import USERS


class MoodleFuse(object):

    def __init__(self, settings=None, testing=None):
        setup(settings)
        self._create_filesystem_root()
        Filesystem(config['LOCAL_MOODLE_FOLDER'], testing)

    def _create_filesystem_root(self):
        moodle_fs_path = config['LOCAL_MOODLE_FOLDER']
        try:
            os.makedirs(moodle_fs_path)
        except OSError as e:
            if e.errno is not errno.EEXIST:
                raise e



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle errors relating to course incidents.
"""

from moodlefuse.moodle.exception import MoodleException


class InvalidMoodleIndex(MoodleException):
    def __init__(self):
        debug_info = "Index page supplied in config is invalid"
        super(InvalidMoodleIndex, self).__init__(debug_info)


class UnableToObtainCourseList(MoodleException):
    def __init__(self):
        debug_info = "Failed to obtain course list from Moodle"
        super(UnableToObtainCourseList, self).__init__(debug_info)


class UnableToOAddCourseCategory(MoodleException):
    def __init__(self):
        debug_info = "Failed to create c ategory in Moodle"
        super(UnableToOAddCourseCategory, self).__init__(debug_info)


class UnableToObtainCategoryList(MoodleException):
    def __init__(self):
        debug_info = "Failed to obtain course category list from Moodle"
        super(UnableToObtainCategoryList, self).__init__(debug_info)



#!/usr/bin/env python
# encoding: utf-8


from functools import wraps

import os


def get_cache_path_based_on_location(location):
    config_folder = os.path.join(os.path.expanduser('~'), '.moodlefuse')
    cache_base = os.path.join(config_folder, 'cache')
    return cache_base + '/' + "_".join(location)


def throws_moodlefuse_error(moodlefuse_error):
    def inner(f):
        def wrapped(*args):
            try:
                return f(*args)
            except Exception:
                raise moodlefuse_error()
        return wraps(f)(wrapped)
    return inner



#!/usr/bin/env python
# encoding: utf-8

import os

from moodlefuse.helpers import get_cache_path_based_on_location
from moodlefuse.filesystem.files.file import File
from moodlefuse.core import config


class CacheFile(File):

    def __init__(self, cache_path):
        super(CacheFile, self).__init__()
        self.path = cache_path
        st = os.lstat(cache_path)

        self.attrs = dict(
            (key, getattr(st, key)) for key in (
                'st_atime',
                'st_ctime',
                'st_gid',
                'st_mode',
                'st_mtime',
                'st_nlink',
                'st_size',
                'st_uid'
            )
        )

    @staticmethod
    def create_file(location, content=None):
        if 'DOWNLOADS' in config:
            cache_path = config['DOWNLOADS'] + '/testfile'
        else:
            cache_path = get_cache_path_based_on_location(location)
        with open(cache_path, 'w') as cache_file:
            if content is not None:
                cache_file.write(content)
            cache_file.close()
            return cache_path



#!/usr/bin/env python
# encoding: utf-8

from moodlefuse.core import config


class PathParser(object):

    @staticmethod
    def is_in_root(location):
        return len(location) is 0

    @staticmethod
    def is_in_course(location):
        return len(location) is 1

    @staticmethod
    def is_in_course_categorie(location):
        return len(location) is 2

    @staticmethod
    def is_file(location):
        return len(location) is 3

    @staticmethod
    def is_assignment(location):
        return len(location) is 4

    @staticmethod
    def is_assignment_submission(location):
        return len(location) is 5

    @staticmethod
    def get_position_in_filesystem_as_array(path):
        path = path.replace(config['LOCAL_MOODLE_FOLDER'] + '/', '')

        if len(path) is 0:
            return []

        path_sections = path.split("/")
        return path_sections



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle course Moodle actions, it extends MoodleHandler
   and therefor is an observer of Moodle.
"""

from moodlefuse.moodle.courses.course_parser import CourseParser
from moodlefuse.moodle.handler import MoodleHandler


class CourseHandler(MoodleHandler):

    def __init__(self, emulator, js_emulator):
        super(self.__class__, self).__init__(emulator, js_emulator)
        self.parser = CourseParser()

    def get_courses_as_array(self):
        course_scraper = self.moodle.get_courses()
        return self.parser.parse_courses(course_scraper)

    def get_course_categories_as_array(self, course):
        course_contents = self.enter_course_and_get_contents(course)
        return self.parser.parse_course_category_titles(course_contents)

    def add_new_category(self, categoryname):
        course_html = self.moodle.set_modify_moodle(modify=True)
        add_section_link = self.parser.get_add_section_link(course_html)
        course_html_with_section = self.moodle.follow_link(add_section_link)
        edit_category_link = self.parser.get_last_sections_edit_button(
            course_html_with_section)
        self.moodle.follow_link_with_js(edit_category_link)
        self.moodle.change_category_name(categoryname)

    def enter_course_and_get_contents(self, course):
        courses_scrapper = self.moodle.get_courses()
        course_link = self.parser.get_course_link(courses_scrapper, course)
        course_page = self.moodle.get_course_contents(course_link)
        return self.parser.parse_course_content_from_page(course_page)

    def enter_course_with_js(self, coursename):
        courses_scrapper = self.moodle.get_courses()
        course_link = self.parser.get_course_link(courses_scrapper, coursename)
        self.moodle.follow_link_with_js(course_link)

    def get_course_category_contents(self, course_content, category):
        return self.parser.parse_course_category_from_course(
            course_content,
            category
        )



import os

DEBUG = False
MOODLE_WEB_ADDRESS = 'file://' + \
                     os.path.join(
                         os.path.dirname(os.path.realpath(__file__)),
                         '../moodle_implementation'
                     ) + '/login/index.html'
MOODLE_INDEX_ADDRESS = 'file://' + \
                     os.path.join(
                         os.path.dirname(os.path.realpath(__file__)),
                         '../moodle_implementation'
                     ) + '/homepage.html'
LOCAL_MOODLE_FOLDER = os.path.join(os.path.expanduser('~'), 'tmp')
USERNAME = 'testuser'
PASSWORD = 'testpassword'
TEST_DATA = 'file://' + os.path.join(
                 os.path.dirname(os.path.realpath(__file__)),
                 '../moodle_implementation'
             )
DOWNLOADS = os.path.join(
                 os.path.dirname(os.path.realpath(__file__)),
                 '../moodle_implementation'
             ) + '/downloads'


from sqlalchemy import Column, String
from moodlefuse.model_manager import Base


class User(Base):
    __tablename__ = 'users'
    username = Column(String(255), primary_key=True)
    password = Column(String(255))

    def __init__(self, username, password):
        self.username = username
        self.password = password



#!/usr/bin/env python
# encoding: utf-8

from moodlefuse.filesystem.file_operations import FileOperationOverrider
from fuse import FUSE


class Filesystem(object):

    def __init__(self, mount, testing=False):
        if not testing:
            FUSE(FileOperationOverrider(mount), mount, foreground=True)



#!/usr/bin/env python
# encoding: utf-8

"""Class to configure MoodleFUSE
"""

import os
import argparse
import getpass

from alembic import command
from ConfigParser import SafeConfigParser
from alembic.config import Config as AlembicConfig

from moodlefuse.models.users import User
from moodlefuse.model_manager import setup_model
from moodlefuse.services import USERS


def main():
    Configurer()


class Configurer(object):

    def __init__(self):
        config_folder = self._create_config_folder()
        self._create_user_database()
        self._create_configuration(config_folder)
        self._create_filesystem_folder()
        self._create_file_cache()

    def _create_config_folder(self):
        config_folder = os.path.join(os.path.expanduser('~'), '.moodlefuse')
        if not os.path.exists(config_folder):
            os.makedirs(config_folder)
        os.chmod(config_folder, 0o700)
        return config_folder

    def _create_file_cache(self):
        cache_folder = os.path.join(
            os.path.expanduser('~'),
            '.moodlefuse/cache')
        if not os.path.exists(cache_folder):
            os.makedirs(cache_folder)
        os.chmod(cache_folder, 0o700)
        return cache_folder

    def _create_configuration(self, config_folder):
        args = self._generate_args()
        profile = args.pop('profile')
        config_file_path = config_folder + '/moodlefuse.conf'
        config = self._modify_config_profile(config_file_path, profile)
        self._add_user_to_database(config, profile)
        config_file = open(config_file_path, 'w+')
        config.write(config_file)

    def _create_user_database(self):
        database_file = os.path.join(
            os.path.expanduser('~'),
            '.moodlefuse/moodlefuse.sqlite'
        )

        if not os.path.exists(database_file):
            directory = os.path.join(os.path.dirname(__file__), '../alembic')
            config = AlembicConfig(os.path.join(
                directory,
                'alembic.ini'
            ))
            config.set_main_option('script_location', directory)
            command.upgrade(config, 'head', sql=False, tag=None)

        setup_model('sqlite:///' + database_file)

    def _generate_args(self):
        parser = argparse.ArgumentParser(
            'Command line utility for configuring moodlefuse'
        )

        parser.add_argument(
            '-p',
            '--profile',
            required=False,
            help='The profile to configure, default is initial',
            default='initial'
        )

        args = parser.parse_args()

        return vars(args)

    def _modify_config_profile(self, config_file, profile):
        config = SafeConfigParser()
        config.read(config_file)

        if not config.has_section(profile):
            config.add_section(profile)

        config = self._set_attributes_of_profile(config, profile)

        return config

    def _set_attributes_of_profile(self, config, profile):
        config = self._set_attribute_of_profile(
            config, profile, 'moodle_web_address',
            'Moodle server address',
            'http://www.moodle.com'
        )

        default_index = config.get(profile, 'moodle_web_address')

        config = self._set_attribute_of_profile(
            config, profile, 'moodle_index_address',
            'Moodle index address',
            default_index
        )

        config = self._set_attribute_of_profile(
            config, profile, 'local_moodle_folder',
            'Local Moodle folder name', 'moodle'
        )

        config = self._set_attribute_of_profile(
            config, profile, 'username',
            'Moodle Username', ''
        )

        return config

    def _get_hidden_password(self, found_user):
        return '*' * len(found_user.password)

    def _add_user_to_database(self, config, profile):
        username = config.get(profile, 'username')

        found_user = USERS.get(User, username)

        password = self._get_new_password_or_return_old(found_user)

        USERS.create(
            username=username,
            password=password
        )

    def _get_new_password_or_return_old(self, found_user):
        password_hidden = ''
        if found_user is not None:
            password_hidden = self._get_hidden_password(found_user)
            USERS.delete(found_user)

        password = self._read_in_config_attribute_or_use_default(
            'Moodle Password',
            password_hidden,
            getpass.getpass
        )

        if password == password_hidden and found_user is not None:
            password = found_user.password

        return password

    def _set_attribute_of_profile(self, config, profile, attribute, message, default):
        if config.has_option(profile, attribute):
            default = config.get(profile, attribute)

        attribute_value = self._read_in_config_attribute_or_use_default(
            message,
            default,
            raw_input
        )

        config.set(profile, attribute, attribute_value)

        return config

    def _read_in_config_attribute_or_use_default(self, message, default, method):
        attribute = method(message + ' [' + default + ']: ')
        if attribute == '':
            attribute = default

        return attribute

    def _create_filesystem_folder(self):
        config_folder = os.path.join(os.path.expanduser('~'), 'moodle')
        if not os.path.exists(config_folder):
            os.makedirs(config_folder)
        os.chmod(config_folder, 0o700)
        return config_folder



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle base Moodle actions.
"""

from moodlefuse.moodle.api import MoodleAPI


class MoodleHandler(object):

    def __init__(self, emulator, js_emulator):
        self.moodle = MoodleAPI(emulator, js_emulator)



#!/usr/bin/env python
# encoding: utf-8

"""Derived class to parse Moodle resource items, it parses
   Moodle and therefor extends off of Parser
"""

from moodlefuse.moodle.parser import Parser


class ResourceParser(Parser):

    def __init__(self):
        super(ResourceParser, self).__init__()

    def parse_course_resources(self, category_content):
        resources_html = self._get_resource_html(category_content)
        return self.scraper.get_instances_from_span_list(resources_html)

    def parse_course_resource_url(self, category_content, filename):
        activity_instances = self.scraper.get_html_items_with_divclass(
            category_content, 'activityinstance')

        return self.scraper.get_link_from_span_list_with_type_and_name(
            activity_instances, 'File', filename
        )

    def parse_course_assignment_url(self, category_content, filename):
        activity_instances = self.scraper.get_html_items_with_divclass(
            category_content, 'activityinstance')

        return self.scraper.get_link_from_span_list_with_type_and_name(
            activity_instances, 'Assignment', filename
        )

    def _get_resource_html(self, category_content):
        return self.scraper.get_html_items_with_spanclass(
            category_content, 'instancename')



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm.exc import NoResultFound

Base = declarative_base()

session = None


def setup_model(database_file):
    global session
    engine = create_engine(database_file)
    s = sessionmaker()
    s.configure(bind=engine)
    Base.metadata.bind = engine
    Base.metadata.create_all(engine)
    session = s()


class Service(object):
    __model__ = None

    def _isinstance(self, model, raise_error=True):
        valid_type = isinstance(model, self.__model__)
        if not valid_type and raise_error:
            raise ValueError('%s is not of type %s' % (model, self.__model__))
        return valid_type

    def get(self, User, primarykey):
        try:
            user_object = session.query(User).filter(User.username == primarykey).one()
            return user_object
        except NoResultFound:
            return None

    def save(self, model):
        self._isinstance(model)
        session.add(model)
        session.commit()
        return model

    def create(self, **kwargs):
        return self.save(self.__model__(**kwargs))

    def delete(self, model):
        self._isinstance(model)
        session.delete(model)
        session.commit()
